package com.seeyon.apps.govdoc.event;

import java.sql.Timestamp;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.seeyon.apps.edoc.event.EdocAddCommentEvent;
import com.seeyon.apps.edoc.event.EdocAppointStepBackEvent;
import com.seeyon.apps.edoc.event.EdocAutoSkipEvent;
import com.seeyon.apps.edoc.event.EdocCancelEvent;
import com.seeyon.apps.edoc.event.EdocDelEvent;
import com.seeyon.apps.edoc.event.EdocFinishEvent;
import com.seeyon.apps.edoc.event.EdocProcessEvent;
import com.seeyon.apps.edoc.event.EdocStepBackEvent;
import com.seeyon.apps.edoc.event.EdocStopEvent;
import com.seeyon.apps.govdoc.bo.DateSharedWithWorkflowEngineThreadLocal;
import com.seeyon.apps.govdoc.util.GovdocParamUtil;
import com.seeyon.apps.govdoc.util.GovdocUtil;
import com.seeyon.apps.govdoc.vo.GovdocBaseVO;
import com.seeyon.apps.govdoc.vo.GovdocDealVO;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.content.comment.Comment;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.event.EventDispatcher;
import com.seeyon.ctp.util.json.JSONUtil;
import com.seeyon.v3x.edoc.domain.EdocSummary;

public class GovdocEventDispatcher {
	
	/**
	 * 公文流程撤销相关事件调用
	 * @param source
	 * @param paramMap
	 */
	public static void fireEventForCancel(Object source, Map<String, Object> paramMap) {
		// 撤销流程事件
		EdocCancelEvent event = new EdocCancelEvent(source);
		event.setSummaryId(GovdocParamUtil.getLong(paramMap, "summaryId"));
		event.setUserId(GovdocParamUtil.getLong(paramMap, "userId"));
		event.setMessage(GovdocParamUtil.getString(paramMap, "repealComment"));
		EventDispatcher.fireEvent(event);
		// 发送消息给督办人，更新督办状态，并删除督办日志、删除督办记录、删除催办次数
	}
	
	/**
	 * 公文处理-多级会签
	 * @param source
	 * @param summary
	 * @param affair
	 */
	public static void fireEventForCurJsonPerm(Object source, EdocSummary summary, CtpAffair affair) {
		GovDocURLEvent event = new GovDocURLEvent(source);
		if (affair != null) {
			event.setAffairId(affair.getId());
		}
		event.setEdocSummary(summary);
		event.setApp(String.valueOf(ApplicationCategoryEnum.edoc.key()));
		event.setSubApp(String.valueOf(summary.getGovdocType()));
		EventDispatcher.fireEvent(event);
	}
	
	/**
	 * 发送自动跳过的事件
	 * @param source
	 */  
	public static void fireAutoSkipEvent(Object source) {	
		Map<String, String> map = DateSharedWithWorkflowEngineThreadLocal.getRepeatAffairs();
		if (map != null && !map.isEmpty()) {
			java.util.Queue<Map<String, String>> affairsQueue = new LinkedList<Map<String, String>>();
			//获取自动跳过的json串
			String skipJsonString = map.get("skipJsonString");
			List<Map<String,String>> skipJsonArray = (List<Map<String, String>>)JSONUtil.parseJSONString(skipJsonString);
			if (skipJsonArray != null && !skipJsonArray.isEmpty()) {
				for (Map<String,String> beanObj : skipJsonArray) {
					Map<String, String> dataMap = GovdocUtil.cloneMap(beanObj);
					dataMap.put("_affairIds", beanObj.get("affairId"));
					dataMap.put("_skipAgentId", beanObj.get("skipAgentId"));
					affairsQueue.offer(dataMap);
				}
			}
			
			EdocAutoSkipEvent event = new EdocAutoSkipEvent(source);
			event.setDataQueues(affairsQueue);
			EventDispatcher.fireEventAfterCommit(event);
		}
	}
	
	@SuppressWarnings("unchecked")
	public static void fireAutoSkipEvent(Object source, Map<String, String> map) {
		if (map != null && !map.isEmpty()) {
			java.util.Queue<Map<String, String>> affairsQueue = new LinkedList<Map<String, String>>();
			//获取自动跳过的json串
			String skipJsonString = map.get("skipJsonString");
			List<Map<String,String>> skipJsonArray = (List<Map<String, String>>) JSONUtil.parseJSONString(skipJsonString);
			if (skipJsonArray != null && !skipJsonArray.isEmpty()) {
				for (Map<String,String> beanObj : skipJsonArray) {
					Map<String, String> dataMap = GovdocUtil.cloneMap(map);
					dataMap.put("_affairIds", beanObj.get("affairId"));
					dataMap.put("_skipAgentId", beanObj.get("skipAgentId"));
					dataMap.put("mergeDealType", beanObj.get("mergeDealType"));
					dataMap.put("_commentId", beanObj.get("_commentId"));
					affairsQueue.offer(dataMap);
				}
			}
			EdocAutoSkipEvent event = new EdocAutoSkipEvent(source);
			event.setDataQueues(affairsQueue);
			EventDispatcher.fireEventAfterCommit(event);
		}		
	}
	
	/**
	 * 流程处理事件通知
	 * @param source
	 * @param dealVo
	 */
	public static void fireEdocProcessEvent(Object source, GovdocBaseVO dealVo) {
		EdocSummary summary = dealVo.getSummary();
		CtpAffair affair = dealVo.getAffair();
		Comment comment = dealVo.getComment();
		//流程处理事件通知
        EdocProcessEvent event = new EdocProcessEvent(source);
        event.setSummaryId(summary.getId());
        event.setAffair(affair);
        event.setComment(comment);
        event.setSenderId(summary.getStartMemberId());
        event.setUserId(AppContext.currentUserId());
        event.setTemplateId(summary.getTempleteId());
        event.setBodyType(summary.getBodyType());
        EventDispatcher.fireEvent(event);
	}
	
	/**
	 * 流程结束事件通知
	 * @param source
	 * @param dealVo
	 */
	public static void fireEdocFinishEvent(Object source, GovdocBaseVO dealVo) {
		Timestamp now = new Timestamp(System.currentTimeMillis());
		CtpAffair affair = dealVo.getAffair();
		if(affair != null) {
			//流程结束事件通知
			EdocFinishEvent event = new EdocFinishEvent(source, affair.getMemberId(), now, affair);
	        EventDispatcher.fireEventAfterCommit(event);
		}
	}
	
	/**
	 * 流程回退事件
	 * @param source
	 * @param dealVo
	 */
	public static void fireEdocStepBackEvent(Object source, GovdocBaseVO dealVo) {
		EdocSummary summary = dealVo.getSummary();
		CtpAffair affair = dealVo.getAffair();
		User user = dealVo.getCurrentUser();
		//流程回退事件
        EdocStepBackEvent backEvent = new EdocStepBackEvent(source);
        backEvent.setSummaryId(summary.getId());
        backEvent.setTemplateId(summary.getTempleteId());
        backEvent.setSenderId(summary.getStartMemberId());
        backEvent.setBodyType(summary.getBodyType());
        backEvent.setUserId(user.getId());
        backEvent.setAffair(affair);
        EventDispatcher.fireEventAfterCommit(backEvent);
	}
	
	/**
	 * 指定回退事件
	 * @param source
	 * @param dealVo
	 */
	public static void fireEdocAppointStepBackEvent(Object source, GovdocDealVO dealVo) {
		EdocSummary summary = dealVo.getSummary();
		User user = dealVo.getCurrentUser();
		// 指定回退事件
		EdocAppointStepBackEvent backEvent = new EdocAppointStepBackEvent(source);
		backEvent.setSelectTargetNodeId(dealVo.getSelectTargetNodeId());
		backEvent.setBodyType(summary.getBodyType());
		backEvent.setSenderId(summary.getStartMemberId());
		backEvent.setUserId(user.getId());
		backEvent.setTemplateId(summary.getTempleteId());
		backEvent.setSummaryId(summary.getId());
		EventDispatcher.fireEventAfterCommit(backEvent);
	}
	
	/**
	 * 流程正常结束通知
	 * @param source
	 * @param dealVo
	 */
	public static void fireEdocStopEvent(Object source, GovdocBaseVO dealVo) {
		EdocSummary summary = dealVo.getSummary();
		User user = dealVo.getCurrentUser();
		//流程正常结束通知
		EdocStopEvent stopEvent = new EdocStopEvent(source);
		stopEvent.setSummaryId(summary.getId());
		stopEvent.setUserId(user.getId());
	    EventDispatcher.fireEvent(stopEvent);
	}
	
	/**
	 * 协同列表删除监听
	 * @param source
	 * @param dealVo
	 */
	public static void fireEdocDelEvent(Object source, GovdocBaseVO dealVo) {
		CtpAffair affair = dealVo.getAffair();
		//协同列表删除监听
		EdocDelEvent takedel = new EdocDelEvent(source);
	    takedel.setAffair(affair);
	    EventDispatcher.fireEvent(takedel);
	}
	
	/**
	 * 增加意见通知
	 * @param source
	 * @param dealVo
	 */
	public static void fireEdocAddCommentEvent(Object source, GovdocBaseVO dealVo) {
		EdocAddCommentEvent commentEvent = new EdocAddCommentEvent(source);
		commentEvent.setCommentId(dealVo.getComment().getId());
		EventDispatcher.fireEvent(commentEvent);
	}
	
}
